<?php
/**
 * Video Url Parser
 *
 * Parses URLs from major cloud video providers. Capable of returning
 * keys from various video embed and link urls to manipulate and
 * access videos in various ways.
 */
class VideoUrlParser
{

	public static function identify_service($url)
	{
		if (preg_match('%youtube|youtu\.be%i', $url)) {
			return 'youtube';
		}
		elseif (preg_match('%vimeo%i', $url)) {
			return 'vimeo';
		}
		elseif (preg_match('/video\/*/', $url)) {
			return 'mp4';
		}
		return null;
	}

	public static function get_url_id($url)
	{
		$service = self::identify_service($url);
		if ($service == 'youtube') {
			return self::get_youtube_id($url);
		}
		elseif ($service == 'vimeo') {
			return self::get_vimeo_id($url);
		}
		return null;
	}

	public static function get_url_embed($url)
	{
		$service = self::identify_service($url);
		$id = self::get_url_id($url);
		if ($service == 'youtube') {
			return self::get_youtube_embed($id);
		}
		elseif ($service == 'vimeo') {
			return self::get_vimeo_embed($id);
		}
		elseif ($service == 'mp4') {
			return $url;
		}
		return null;
	}

	public static function get_youtube_id($url)
	{
		$youtube_url_keys = array('v','vi');
		// Try to get ID from url parameters
		$key_from_params = self::parse_url_for_params($url, $youtube_url_keys);
		if ($key_from_params) return $key_from_params;
		// Try to get ID from last portion of url
		return self::parse_url_for_last_element($url);
	}

	public static function get_youtube_embed($youtube_video_id, $autoplay = 1)
	{
		$embed = "https://youtube.com/embed/$youtube_video_id?rel=0&amp;controls=0&amp;showinfo=0";
		return $embed;
	}

	public static function get_vimeo_id($url)
	{
		// Try to get ID from last portion of url
		return self::parse_url_for_last_element($url);
	}

	public static function get_vimeo_embed($vimeo_video_id, $autoplay = 1)
	{
		$embed = "https://player.vimeo.com/video/$vimeo_video_id?byline=0&amp;portrait=0&amp";
		return $embed;
	}

	public static function get_cover($url)
	{
		$service = self::identify_service($url);
		if ($service == 'youtube') {
			return 'https://img.youtube.com/vi/'.self::get_youtube_id($url).'/maxresdefault.jpg';
		}
		elseif ($service == 'vimeo') {
			$hash = unserialize(file_get_contents('http://vimeo.com/api/v2/video/'.self::get_vimeo_id($url).'.php'));
			return $hash[0]['thumbnail_large'];
		}
		return null;
	}

	public static function get_background_video($url)
	{
		$service = self::identify_service($url);
		if($service == 'mp4') {
			return '<video class="jquery-background-video is-playing" playsinline autoplay muted loop src="'.esc_url($url).'" type="video/mp4"></video>';
		} else if($service == 'youtube') {
			return '<div class="video"><iframe frameborder="0" height="100%" width="100%" src="'.esc_url('https://youtube.com/embed/'.self::get_url_id($url).'?autoplay=1&controls=0&showinfo=0&autohide=1&mute=1&loop=1&enablejsapi=1&playlist='.self::get_url_id($url)).'"></iframe></div>';
		} else if($service == 'vimeo') {
			return '<div class="video"><iframe frameborder="0" height="100%" width="100%" src="'.esc_url('https://player.vimeo.com/video/'.self::get_url_id($url).'?autoplay=1&loop=1&autopause=0&background=1').'"></iframe></div>';
		}
		return null;
	}

	public static function get_player($url)
	{
		$service = self::identify_service($url);
		$id = self::get_url_id($url);
		if($service == 'mp4') {
			return '<video controls class="pswp__video" width="1920" height="1080" src="'.esc_url($url).'" type="video/mp4"></video>';
		} else if($service == 'youtube') {
			return '<iframe class="pswp__video" width="1920" height="1080" src="'.esc_url(self::get_youtube_embed($id)).'" frameborder="0" allowfullscreen=""></iframe>';
		} else if($service == 'vimeo') {
			return '<iframe class="pswp__video" width="1920" height="1080" src="'.esc_url(self::get_vimeo_embed($id)).'" frameborder="0" allowfullscreen=""></iframe>';
		}
		return null;
	}

	public static function get_time_format($seconds) 
	{
		if($seconds >= 3600) {
			return gmdate("H:i:s", $seconds);
		} else {
			return gmdate("i:s", $seconds);
		}
		return null;
	}

	public static function get_duration($url)
	{
		$service = self::identify_service($url);
		if ($service == 'youtube') {
			$videoDetails = file_get_contents("https://www.googleapis.com/youtube/v3/videos?id=".self::get_youtube_id($url)."&part=contentDetails,statistics&key=AIzaSyBSW6ZP4p7njnMq_QbgV2nK5L-vfK7zCB4");
			$VidDuration = json_decode($videoDetails, true);
			//var_dump($VidDuration);
			foreach ($VidDuration['items'] as $vidTime) {
				$VidDuration = $vidTime['contentDetails']['duration'];
			}
			$pattern = '/PT(\d+)M(\d+)S/';
			preg_match($pattern,$VidDuration,$matches);
			$seconds = $matches[1]*60+$matches[2];
			$duration = self::get_time_format($seconds);
			return $duration;
		}
		elseif ($service == 'vimeo') {
			$json_url = 'http://vimeo.com/api/v2/video/' . self::get_vimeo_id($url) . '.xml';
			if(!function_exists('curl_init')) return false;
			$ch = curl_init($json_url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HEADER, 0);
			$data = curl_exec($ch);
			curl_close($ch);
			$data = new SimpleXmlElement($data, LIBXML_NOCDATA);

			if (!isset($data->video->duration)) {
				return false;
			}

			$seconds = intval($data->video->duration);
			$duration = self::get_time_format($seconds);
			return $duration;
		}
	}

	private static function parse_url_for_params($url, $target_params)
	{
		parse_str( parse_url( $url, PHP_URL_QUERY ), $my_array_of_params );
		foreach ($target_params as $target) {
			if (array_key_exists ($target, $my_array_of_params)) {
				return $my_array_of_params[$target];
			}
		}
		return null;
	}

	private static function parse_url_for_last_element($url)
	{
		$url_parts = explode("/", $url);
		$prospect = end($url_parts);
		$prospect_and_params = preg_split("/(\?|\=|\&)/", $prospect);
		if ($prospect_and_params) {
			return $prospect_and_params[0];
		} else {
			return $prospect;
		}
		return $url;
	}
}
 ?>